from production.command import ProductionCommand


class DevicesUpdatedCommand(ProductionCommand):
    def __init__(self):
        super(DevicesUpdatedCommand, self).__init__(None, None)

    def get_hash_extra_value(
        self,   # type: DevicesUpdatedCommand
    ):
        return "DevicesUpdatedCommand"

