from abc import (
    ABCMeta,
    abstractmethod,
)
from typing import Dict


class ProductRepository(object):
    __metaclass__ = ABCMeta

    @abstractmethod
    def get_order_items_names(
        self,
        order_items,    # type: str
    ):
        # type: (...) -> Dict

        raise NotImplementedError
