data = (
'ggyeols',    # 0x00
'ggyeolt',    # 0x01
'ggyeolp',    # 0x02
'ggyeolh',    # 0x03
'ggyeom',    # 0x04
'ggyeob',    # 0x05
'ggyeobs',    # 0x06
'ggyeos',    # 0x07
'ggyeoss',    # 0x08
'ggyeong',    # 0x09
'ggyeoj',    # 0x0a
'ggyeoc',    # 0x0b
'ggyeok',    # 0x0c
'ggyeot',    # 0x0d
'ggyeop',    # 0x0e
'ggyeoh',    # 0x0f
'ggye',    # 0x10
'ggyeg',    # 0x11
'ggyegg',    # 0x12
'ggyegs',    # 0x13
'ggyen',    # 0x14
'ggyenj',    # 0x15
'ggyenh',    # 0x16
'ggyed',    # 0x17
'ggyel',    # 0x18
'ggyelg',    # 0x19
'ggyelm',    # 0x1a
'ggyelb',    # 0x1b
'ggyels',    # 0x1c
'ggyelt',    # 0x1d
'ggyelp',    # 0x1e
'ggyelh',    # 0x1f
'ggyem',    # 0x20
'ggyeb',    # 0x21
'ggyebs',    # 0x22
'ggyes',    # 0x23
'ggyess',    # 0x24
'ggyeng',    # 0x25
'ggyej',    # 0x26
'ggyec',    # 0x27
'ggyek',    # 0x28
'ggyet',    # 0x29
'ggyep',    # 0x2a
'ggyeh',    # 0x2b
'ggo',    # 0x2c
'ggog',    # 0x2d
'ggogg',    # 0x2e
'ggogs',    # 0x2f
'ggon',    # 0x30
'ggonj',    # 0x31
'ggonh',    # 0x32
'ggod',    # 0x33
'ggol',    # 0x34
'ggolg',    # 0x35
'ggolm',    # 0x36
'ggolb',    # 0x37
'ggols',    # 0x38
'ggolt',    # 0x39
'ggolp',    # 0x3a
'ggolh',    # 0x3b
'ggom',    # 0x3c
'ggob',    # 0x3d
'ggobs',    # 0x3e
'ggos',    # 0x3f
'ggoss',    # 0x40
'ggong',    # 0x41
'ggoj',    # 0x42
'ggoc',    # 0x43
'ggok',    # 0x44
'ggot',    # 0x45
'ggop',    # 0x46
'ggoh',    # 0x47
'ggwa',    # 0x48
'ggwag',    # 0x49
'ggwagg',    # 0x4a
'ggwags',    # 0x4b
'ggwan',    # 0x4c
'ggwanj',    # 0x4d
'ggwanh',    # 0x4e
'ggwad',    # 0x4f
'ggwal',    # 0x50
'ggwalg',    # 0x51
'ggwalm',    # 0x52
'ggwalb',    # 0x53
'ggwals',    # 0x54
'ggwalt',    # 0x55
'ggwalp',    # 0x56
'ggwalh',    # 0x57
'ggwam',    # 0x58
'ggwab',    # 0x59
'ggwabs',    # 0x5a
'ggwas',    # 0x5b
'ggwass',    # 0x5c
'ggwang',    # 0x5d
'ggwaj',    # 0x5e
'ggwac',    # 0x5f
'ggwak',    # 0x60
'ggwat',    # 0x61
'ggwap',    # 0x62
'ggwah',    # 0x63
'ggwae',    # 0x64
'ggwaeg',    # 0x65
'ggwaegg',    # 0x66
'ggwaegs',    # 0x67
'ggwaen',    # 0x68
'ggwaenj',    # 0x69
'ggwaenh',    # 0x6a
'ggwaed',    # 0x6b
'ggwael',    # 0x6c
'ggwaelg',    # 0x6d
'ggwaelm',    # 0x6e
'ggwaelb',    # 0x6f
'ggwaels',    # 0x70
'ggwaelt',    # 0x71
'ggwaelp',    # 0x72
'ggwaelh',    # 0x73
'ggwaem',    # 0x74
'ggwaeb',    # 0x75
'ggwaebs',    # 0x76
'ggwaes',    # 0x77
'ggwaess',    # 0x78
'ggwaeng',    # 0x79
'ggwaej',    # 0x7a
'ggwaec',    # 0x7b
'ggwaek',    # 0x7c
'ggwaet',    # 0x7d
'ggwaep',    # 0x7e
'ggwaeh',    # 0x7f
'ggoe',    # 0x80
'ggoeg',    # 0x81
'ggoegg',    # 0x82
'ggoegs',    # 0x83
'ggoen',    # 0x84
'ggoenj',    # 0x85
'ggoenh',    # 0x86
'ggoed',    # 0x87
'ggoel',    # 0x88
'ggoelg',    # 0x89
'ggoelm',    # 0x8a
'ggoelb',    # 0x8b
'ggoels',    # 0x8c
'ggoelt',    # 0x8d
'ggoelp',    # 0x8e
'ggoelh',    # 0x8f
'ggoem',    # 0x90
'ggoeb',    # 0x91
'ggoebs',    # 0x92
'ggoes',    # 0x93
'ggoess',    # 0x94
'ggoeng',    # 0x95
'ggoej',    # 0x96
'ggoec',    # 0x97
'ggoek',    # 0x98
'ggoet',    # 0x99
'ggoep',    # 0x9a
'ggoeh',    # 0x9b
'ggyo',    # 0x9c
'ggyog',    # 0x9d
'ggyogg',    # 0x9e
'ggyogs',    # 0x9f
'ggyon',    # 0xa0
'ggyonj',    # 0xa1
'ggyonh',    # 0xa2
'ggyod',    # 0xa3
'ggyol',    # 0xa4
'ggyolg',    # 0xa5
'ggyolm',    # 0xa6
'ggyolb',    # 0xa7
'ggyols',    # 0xa8
'ggyolt',    # 0xa9
'ggyolp',    # 0xaa
'ggyolh',    # 0xab
'ggyom',    # 0xac
'ggyob',    # 0xad
'ggyobs',    # 0xae
'ggyos',    # 0xaf
'ggyoss',    # 0xb0
'ggyong',    # 0xb1
'ggyoj',    # 0xb2
'ggyoc',    # 0xb3
'ggyok',    # 0xb4
'ggyot',    # 0xb5
'ggyop',    # 0xb6
'ggyoh',    # 0xb7
'ggu',    # 0xb8
'ggug',    # 0xb9
'ggugg',    # 0xba
'ggugs',    # 0xbb
'ggun',    # 0xbc
'ggunj',    # 0xbd
'ggunh',    # 0xbe
'ggud',    # 0xbf
'ggul',    # 0xc0
'ggulg',    # 0xc1
'ggulm',    # 0xc2
'ggulb',    # 0xc3
'gguls',    # 0xc4
'ggult',    # 0xc5
'ggulp',    # 0xc6
'ggulh',    # 0xc7
'ggum',    # 0xc8
'ggub',    # 0xc9
'ggubs',    # 0xca
'ggus',    # 0xcb
'gguss',    # 0xcc
'ggung',    # 0xcd
'gguj',    # 0xce
'gguc',    # 0xcf
'gguk',    # 0xd0
'ggut',    # 0xd1
'ggup',    # 0xd2
'gguh',    # 0xd3
'ggweo',    # 0xd4
'ggweog',    # 0xd5
'ggweogg',    # 0xd6
'ggweogs',    # 0xd7
'ggweon',    # 0xd8
'ggweonj',    # 0xd9
'ggweonh',    # 0xda
'ggweod',    # 0xdb
'ggweol',    # 0xdc
'ggweolg',    # 0xdd
'ggweolm',    # 0xde
'ggweolb',    # 0xdf
'ggweols',    # 0xe0
'ggweolt',    # 0xe1
'ggweolp',    # 0xe2
'ggweolh',    # 0xe3
'ggweom',    # 0xe4
'ggweob',    # 0xe5
'ggweobs',    # 0xe6
'ggweos',    # 0xe7
'ggweoss',    # 0xe8
'ggweong',    # 0xe9
'ggweoj',    # 0xea
'ggweoc',    # 0xeb
'ggweok',    # 0xec
'ggweot',    # 0xed
'ggweop',    # 0xee
'ggweoh',    # 0xef
'ggwe',    # 0xf0
'ggweg',    # 0xf1
'ggwegg',    # 0xf2
'ggwegs',    # 0xf3
'ggwen',    # 0xf4
'ggwenj',    # 0xf5
'ggwenh',    # 0xf6
'ggwed',    # 0xf7
'ggwel',    # 0xf8
'ggwelg',    # 0xf9
'ggwelm',    # 0xfa
'ggwelb',    # 0xfb
'ggwels',    # 0xfc
'ggwelt',    # 0xfd
'ggwelp',    # 0xfe
'ggwelh',    # 0xff
)
