from typing import (
    Dict,
    List,
)

from nreports_api.shared import JSON


class Line(Dict[str, JSON]):
    pass


class ReportDto(List[Line]):

    def __init__(
        self,   # type: ReportDto
        *args
    ):
        # type: (...) -> None
        super().__init__([Line(line) for line in args[0]])
