from helper import ExtendedEnum
from messagebus import (
    TokenCreator,
    TokenPriority,
)

DEVICE_MANAGER_GROUP = "D"


class ListenedTokens(ExtendedEnum):
    # Processors
    TK_GET_USED_SERVICE = TokenCreator.create_token(
        priority=TokenPriority.low,
        message_group=DEVICE_MANAGER_GROUP,
        number="1",
    )
    TK_GET_DEVICES = TokenCreator.create_token(
        priority=TokenPriority.low,
        message_group=DEVICE_MANAGER_GROUP,
        number="2",
    )
    TK_UPDATE_DEVICE_STATUS = TokenCreator.create_token(
        priority=TokenPriority.low,
        message_group=DEVICE_MANAGER_GROUP,
        number="3",
    )
