from ._LoyaltyTranslatedMessage import LoyaltyTranslatedMessage


class LoyaltyTotaledMessage(object):
    def __init__(
        self,                        # type: LoyaltyTotaledMessage
        loyalty_translated_message,  # type: LoyaltyTranslatedMessage
    ):
        # type: (...) -> None

        self.totalization_message = loyalty_translated_message
