data = (
'ssal',    # 0x00
'ssalg',    # 0x01
'ssalm',    # 0x02
'ssalb',    # 0x03
'ssals',    # 0x04
'ssalt',    # 0x05
'ssalp',    # 0x06
'ssalh',    # 0x07
'ssam',    # 0x08
'ssab',    # 0x09
'ssabs',    # 0x0a
'ssas',    # 0x0b
'ssass',    # 0x0c
'ssang',    # 0x0d
'ssaj',    # 0x0e
'ssac',    # 0x0f
'ssak',    # 0x10
'ssat',    # 0x11
'ssap',    # 0x12
'ssah',    # 0x13
'ssae',    # 0x14
'ssaeg',    # 0x15
'ssaegg',    # 0x16
'ssaegs',    # 0x17
'ssaen',    # 0x18
'ssaenj',    # 0x19
'ssaenh',    # 0x1a
'ssaed',    # 0x1b
'ssael',    # 0x1c
'ssaelg',    # 0x1d
'ssaelm',    # 0x1e
'ssaelb',    # 0x1f
'ssaels',    # 0x20
'ssaelt',    # 0x21
'ssaelp',    # 0x22
'ssaelh',    # 0x23
'ssaem',    # 0x24
'ssaeb',    # 0x25
'ssaebs',    # 0x26
'ssaes',    # 0x27
'ssaess',    # 0x28
'ssaeng',    # 0x29
'ssaej',    # 0x2a
'ssaec',    # 0x2b
'ssaek',    # 0x2c
'ssaet',    # 0x2d
'ssaep',    # 0x2e
'ssaeh',    # 0x2f
'ssya',    # 0x30
'ssyag',    # 0x31
'ssyagg',    # 0x32
'ssyags',    # 0x33
'ssyan',    # 0x34
'ssyanj',    # 0x35
'ssyanh',    # 0x36
'ssyad',    # 0x37
'ssyal',    # 0x38
'ssyalg',    # 0x39
'ssyalm',    # 0x3a
'ssyalb',    # 0x3b
'ssyals',    # 0x3c
'ssyalt',    # 0x3d
'ssyalp',    # 0x3e
'ssyalh',    # 0x3f
'ssyam',    # 0x40
'ssyab',    # 0x41
'ssyabs',    # 0x42
'ssyas',    # 0x43
'ssyass',    # 0x44
'ssyang',    # 0x45
'ssyaj',    # 0x46
'ssyac',    # 0x47
'ssyak',    # 0x48
'ssyat',    # 0x49
'ssyap',    # 0x4a
'ssyah',    # 0x4b
'ssyae',    # 0x4c
'ssyaeg',    # 0x4d
'ssyaegg',    # 0x4e
'ssyaegs',    # 0x4f
'ssyaen',    # 0x50
'ssyaenj',    # 0x51
'ssyaenh',    # 0x52
'ssyaed',    # 0x53
'ssyael',    # 0x54
'ssyaelg',    # 0x55
'ssyaelm',    # 0x56
'ssyaelb',    # 0x57
'ssyaels',    # 0x58
'ssyaelt',    # 0x59
'ssyaelp',    # 0x5a
'ssyaelh',    # 0x5b
'ssyaem',    # 0x5c
'ssyaeb',    # 0x5d
'ssyaebs',    # 0x5e
'ssyaes',    # 0x5f
'ssyaess',    # 0x60
'ssyaeng',    # 0x61
'ssyaej',    # 0x62
'ssyaec',    # 0x63
'ssyaek',    # 0x64
'ssyaet',    # 0x65
'ssyaep',    # 0x66
'ssyaeh',    # 0x67
'sseo',    # 0x68
'sseog',    # 0x69
'sseogg',    # 0x6a
'sseogs',    # 0x6b
'sseon',    # 0x6c
'sseonj',    # 0x6d
'sseonh',    # 0x6e
'sseod',    # 0x6f
'sseol',    # 0x70
'sseolg',    # 0x71
'sseolm',    # 0x72
'sseolb',    # 0x73
'sseols',    # 0x74
'sseolt',    # 0x75
'sseolp',    # 0x76
'sseolh',    # 0x77
'sseom',    # 0x78
'sseob',    # 0x79
'sseobs',    # 0x7a
'sseos',    # 0x7b
'sseoss',    # 0x7c
'sseong',    # 0x7d
'sseoj',    # 0x7e
'sseoc',    # 0x7f
'sseok',    # 0x80
'sseot',    # 0x81
'sseop',    # 0x82
'sseoh',    # 0x83
'sse',    # 0x84
'sseg',    # 0x85
'ssegg',    # 0x86
'ssegs',    # 0x87
'ssen',    # 0x88
'ssenj',    # 0x89
'ssenh',    # 0x8a
'ssed',    # 0x8b
'ssel',    # 0x8c
'sselg',    # 0x8d
'sselm',    # 0x8e
'sselb',    # 0x8f
'ssels',    # 0x90
'sselt',    # 0x91
'sselp',    # 0x92
'sselh',    # 0x93
'ssem',    # 0x94
'sseb',    # 0x95
'ssebs',    # 0x96
'sses',    # 0x97
'ssess',    # 0x98
'sseng',    # 0x99
'ssej',    # 0x9a
'ssec',    # 0x9b
'ssek',    # 0x9c
'sset',    # 0x9d
'ssep',    # 0x9e
'sseh',    # 0x9f
'ssyeo',    # 0xa0
'ssyeog',    # 0xa1
'ssyeogg',    # 0xa2
'ssyeogs',    # 0xa3
'ssyeon',    # 0xa4
'ssyeonj',    # 0xa5
'ssyeonh',    # 0xa6
'ssyeod',    # 0xa7
'ssyeol',    # 0xa8
'ssyeolg',    # 0xa9
'ssyeolm',    # 0xaa
'ssyeolb',    # 0xab
'ssyeols',    # 0xac
'ssyeolt',    # 0xad
'ssyeolp',    # 0xae
'ssyeolh',    # 0xaf
'ssyeom',    # 0xb0
'ssyeob',    # 0xb1
'ssyeobs',    # 0xb2
'ssyeos',    # 0xb3
'ssyeoss',    # 0xb4
'ssyeong',    # 0xb5
'ssyeoj',    # 0xb6
'ssyeoc',    # 0xb7
'ssyeok',    # 0xb8
'ssyeot',    # 0xb9
'ssyeop',    # 0xba
'ssyeoh',    # 0xbb
'ssye',    # 0xbc
'ssyeg',    # 0xbd
'ssyegg',    # 0xbe
'ssyegs',    # 0xbf
'ssyen',    # 0xc0
'ssyenj',    # 0xc1
'ssyenh',    # 0xc2
'ssyed',    # 0xc3
'ssyel',    # 0xc4
'ssyelg',    # 0xc5
'ssyelm',    # 0xc6
'ssyelb',    # 0xc7
'ssyels',    # 0xc8
'ssyelt',    # 0xc9
'ssyelp',    # 0xca
'ssyelh',    # 0xcb
'ssyem',    # 0xcc
'ssyeb',    # 0xcd
'ssyebs',    # 0xce
'ssyes',    # 0xcf
'ssyess',    # 0xd0
'ssyeng',    # 0xd1
'ssyej',    # 0xd2
'ssyec',    # 0xd3
'ssyek',    # 0xd4
'ssyet',    # 0xd5
'ssyep',    # 0xd6
'ssyeh',    # 0xd7
'sso',    # 0xd8
'ssog',    # 0xd9
'ssogg',    # 0xda
'ssogs',    # 0xdb
'sson',    # 0xdc
'ssonj',    # 0xdd
'ssonh',    # 0xde
'ssod',    # 0xdf
'ssol',    # 0xe0
'ssolg',    # 0xe1
'ssolm',    # 0xe2
'ssolb',    # 0xe3
'ssols',    # 0xe4
'ssolt',    # 0xe5
'ssolp',    # 0xe6
'ssolh',    # 0xe7
'ssom',    # 0xe8
'ssob',    # 0xe9
'ssobs',    # 0xea
'ssos',    # 0xeb
'ssoss',    # 0xec
'ssong',    # 0xed
'ssoj',    # 0xee
'ssoc',    # 0xef
'ssok',    # 0xf0
'ssot',    # 0xf1
'ssop',    # 0xf2
'ssoh',    # 0xf3
'sswa',    # 0xf4
'sswag',    # 0xf5
'sswagg',    # 0xf6
'sswags',    # 0xf7
'sswan',    # 0xf8
'sswanj',    # 0xf9
'sswanh',    # 0xfa
'sswad',    # 0xfb
'sswal',    # 0xfc
'sswalg',    # 0xfd
'sswalm',    # 0xfe
'sswalb',    # 0xff
)
