class Tender(object):

    def __init__(
        self,
        tender_id=None,     # type: int
        amount=None,        # type: float
    ):
        # type: (...) -> None

        self.tender_id = tender_id or -1
        self.amount = amount or 0.0
